\name{icustom}
\alias{icustom.plot}
\alias{icustom}
\title{Create a custom interactive plot template}
\description{
  \code{icustom.plot} creates a template of a new, interactive custom
  plot. The resulting function can be used to create instance of the
  new plot.
}
\usage{
icustom.plot(name, construct, preflight, postflight, min.data.dim=1,
             param=list())
}
\arguments{
 \item{name}{name of the custom plot. It can be an arbitrary string
 identifying the plot type. It is also used as the plot title (unless
 changed in preflight by \code{title <<- ...}).}
 \item{construct}{the constructor function (see details).}
 \item{preflight}{optional instance data checking function}
 \item{postflight}{optional instance initialization function}
 \item{min.data.dim}{minimal data dimension}
 \item{param}{list of parameters that will be passed to the instance}
}
\value{
 \code{icustom.plot} returns a function that can be used to create
 instances of the custom plot.
}
\details{
 A custom interactive plot is a plot based on the iPlots package which
 can be customized to display data in an arbitrary way. Each iPlot
 consists of graphical objects that represent data or statistical
 properties (e.g. bars, polygons, lines, points etc.). Those
 `statistical objects' (\code{sobj}) form the primitives of the
 plot and they respond to user interactions automatically (selection,
 highlighting etc.). Beyond statistical object the plot can contain
 any number of iObjects as defined in the \code{iplots} package.

 An interactive custom plot template is defined by specifying the
 statistical objects for given data. This is done using the
 \code{construct} function which is called whenever the underlying
 data or the size of the plot change. The purpose of this function is
 to create \code{sobj} and iObjects as desired. This definition is
 unique to a given plot type (e.g. a histogram) but it in invariant to
 the data supplied. Hence it is defined in the template which can be
 subsequently used to generate any number of interactive custom plots
 of the same type.

 The \code{construct} function should have the signature
 \code{function(width, height, ...)} where \code{width} and
 \code{height} are passed according to the plot size (in pixels) at
 the time of the call (also see discussion below).

 The \code{preflight} function is optional, should have signature
 \code{function(data, env, ...)} and allows the designer to modify
 input data or impose constraints on the data and parameters. When
 used it should return \code{NULL} if no modifications are necessary,
 otherwise the data will be substituted by its return value.

 The \code{postflight} function is optional, should have signature
 \code{function(data, plot, env, ...)} where \code{data}, \code{plot}
 and \code{env} are as described below and \code{...} contains
 pass-through aruments from the instantiation call. It can be used to
 modify initial plot properties and/or initialize the custom
 enviroment.

 \code{param} is an optional list of parameters that will be defined
 for all instances of the plot.

 The \code{construct} function can optionally define axes via
 \code{\link{iaxis}} to help to map data coordinates to screen
 coordinates. In addition to its parameters the following objects are
 available in the scope of the function:
 \itemize{
   \item \code{plot} the iPlot object of the current plot
   \item \code{data} the data used in the plot
   \item \code{env} persistent environment associated with the
 plot. It can be used to store information across calls to
 \code{construct}.
   \item parameters - all parameters from the \code{param} list as
 well as those supplied to the template function call (in that order
 of precedence).
 }
}
\examples{
\dontrun{
# define a new interactive plot type - age pyramid for two genders
iagepyr <- icustom.plot(name = "Age pyramid",
                        min.data.dim = 2, # at least two variables
                        param = list(bin.width = 10), # default bin width
                        construct = function(width, height, ...) {

    side <- unclass(data[[2]])
    left <- which(side==1)
    right <- which(side==2)
    anchor <- min(data[[1]])
    bins <- ceiling(max(data[[1]])/bin.width)
    bin <- ceiling((data[[1]]-anchor)/bin.width)
    l.largest.bin <- max(table(bin));
    r.largest.bin <- l.largest.bin;
    
    # define axes - right, leftt and y
    iaxis.reset.all()
    a.r <- iaxis("cont", "x", c(0, r.largest.bin),
                 c(width/2, width-10), trl=height-10)
    a.l <- iaxis("cont", "x", c(0, l.largest.bin),
                 c(width/2, 10), trl=height-10)
    a.y <- iaxis("cont", "y", c(0, bins),
                 c(height-10,20), trl=width/2)
    
    # define bars for each gender and bin
    sobj.reset()
    for (i in 1:bins) {
      ids.l <- which(bin==i & side==1)
      ids.r <- which(bin==i & side==2)
      
      if (length(ids.l))
        sobj.rect(ids.l, a.l(0), a.y(i-0.5),
                  a.l(length(ids.l))-a.l(0),
                  a.y(i+0.5)-a.y(i-0.5),
                  "left")
      if(length(ids.r)>0)
        sobj.rect(ids.r, a.r(0), a.y(i-0.5),
                  a.r(length(ids.r))-a.r(0),
                  a.y(i+0.5)-a.y(i-0.5),
                  "right")
    }

                        })

# usage of the new plot
iagepyr(data)
}
}
\keyword{hplot}
